/*    Copyright 2009 10gen Inc.
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

#include "mongo/pch.h"

#include "mongo/util/text.h"

#include "mongo/util/startup_test.h"
#include "mongo/util/mongoutils/str.h"
#include <boost/smart_ptr/scoped_array.hpp>

namespace mongo {
    struct TextUnitTest : public StartupTest {
        void run() {
            verify( parseLL("123") == 123 );
            verify( parseLL("-123000000000") == -123000000000LL );
        }
    } textUnitTest;

}  // namespace mongo
